@@ -53,6 +53,7 @@ contains
 
         if (option%ErrSol == 1) then
             call BPACK_Test_Solve_error(bmat, msh%idxe - msh%idxs + 1, option, ptree, stats)
+            if(ptree%MyID==Main_ID)write(*,*) 'RedistV time: ', stats%Time_RedistV
         endif
 
     end subroutine BPACK_Factorization
@@ -66,7 +67,7 @@ contains
         integer rank, index_near, m, n, length, flag, itemp
         real T0
         real(kind=8)::rtemp = 0
-        real(kind=8) tmpfact
+        real(kind=8) tmpfact, tol_used
         real(kind=8) Memory, Memory_near
         integer, allocatable:: index_old(:), index_new(:)
         integer::block_num, block_num_new, level_butterfly
@@ -95,19 +96,24 @@ contains
         if (ptree%MyID == Main_ID .and. option%verbosity >= 0) write (*, *) 'Computing block inverse at level Maxlevel+1...'
         level_c = ho_bf1%Maxlevel + 1
         do ii = ho_bf1%levels(level_c)%Bidxs, ho_bf1%levels(level_c)%Bidxe
-
-         ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat = ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%fullmat
-            nn = size(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat, 1)
-
+#if HAVE_ZFP
+            if(option%use_zfp==1)call ZFP_Decompress(ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%fullmat,ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%fullmatZFP, ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%M, ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%N, tol_used,1)
+#endif
+            nn = size(ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%fullmat, 1)
+            allocate(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat(nn,nn))
+            ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat = ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%fullmat
+#if HAVE_ZFP
+            if(option%use_zfp==1)call ZFP_Compress(ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%fullmat,ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%FullmatZFP,ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%M,ho_bf1%levels(level_c)%BP(ii)%LL(1)%matrices_block(1)%N, option%tol_comp,1)
+#endif
             allocate(Singular(nn))
             allocate(UU(nn,nn))
             allocate(VV(nn,nn))
             call gesvd_robust(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat, Singular, UU, VV, nn, nn, nn)
-            if(Singular(nn)/Singular(1)<BPACK_Jitter)then
+            if(Singular(nn)/Singular(1)<option%jitter)then
                 norm = Singular(1)
-                ! write(*,*)norm*BPACK_Jitter,norm
+                ! write(*,*)norm*option%jitter,norm
                 do iii=1,nn
-                    ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat(iii,iii)=ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat(iii,iii)+norm*BPACK_Jitter
+                    ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat(iii,iii)=ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat(iii,iii)+norm*option%jitter
                 enddo
             endif
             deallocate(UU)
@@ -133,9 +139,19 @@ contains
             !!!!!!! the forward block BP can be deleted if not used in solution phase
 
             ! write(*,*)fnorm(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat,nn,nn)
-
-stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat)/1024.0d3
-
+#if HAVE_ZFP
+            if(option%use_zfp==1)then
+                call ZFP_Compress(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat,ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%FullmatZFP,ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%M,ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%N, option%tol_comp,0)
+                stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%FullmatZFP%buffer_r)/1024.0d3
+#if DAT==0 || DAT==2
+                stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%FullmatZFP%buffer_i)/1024.0d3
+#endif
+            else
+                stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat)/1024.0d3
+            endif
+#else
+            stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_inverse(ii)%LL(1)%matrices_block(1)%fullmat)/1024.0d3
+#endif
         end do
 
         call MPI_barrier(ptree%Comm, ierr)
@@ -387,7 +403,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
             do j = 1, h_mat%myAcols
             do i = 1, h_mat%myArows
                 blocks => h_mat%Local_blocks(j, i)
-                ! call pack_all_blocks_one_node(blocks, msh)
+                ! call pack_all_blocks_one_node(blocks, msh,option)
                 mypgno = blocks%pgno
             enddo
             enddo
@@ -397,7 +413,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
             allocate (h_mat%Computing_matricesblock_u(max(1,h_mat%myAcols), max(1,h_mat%myArows)))
 
 
-            call blacs_gridinfo(ptree%pgrp(1)%ctxt, nprow, npcol, myrow, mycol)
+            call blacs_gridinfo_wrp(ptree%pgrp(1)%ctxt, nprow, npcol, myrow, mycol)
             num_blocks = 2**h_mat%Dist_level
             do kk=1,num_blocks
                 if(ptree%MyID==Main_ID .and. option%verbosity>=0)write(*,*) "starting panel ", kk
@@ -416,7 +432,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
                     blocks => h_mat%Local_blocks(myj, myi)
                     ! call unpack_all_blocks_one_node(blocks, h_mat%Maxlevel, ptree, msh, mypgno)
                     call Hmat_LU(blocks, h_mat, option, stats, ptree, msh)
-                    call pack_all_blocks_one_node(blocks, msh)
+                    call pack_all_blocks_one_node(blocks, msh, option)
                     send=1
                 endif
 
@@ -450,7 +466,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
                 stats%Time_idle = stats%Time_idle + T4 - T3
 
                 if (recv == 1)then
-                    call unpack_all_blocks_one_node(blocks_r, h_mat%Maxlevel, ptree, msh, mypgno)
+                    call unpack_all_blocks_one_node(blocks_r, h_mat%Maxlevel, ptree, msh, mypgno, option)
                     Memory=0
                     call Hmat_block_ComputeMemory(blocks_r, Memory)
                     call LogMemory(stats, Memory)
@@ -461,7 +477,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
                         blocks_uu => h_mat%Local_blocks(myj1, myi)
                         blocks_mm => h_mat%Computing_matricesblock_m(1, 1)
                         call Hmat_LXM(blocks_mm, blocks_uu, h_mat, option, stats, ptree, msh)
-                        call pack_all_blocks_one_node(blocks_uu, msh)
+                        call pack_all_blocks_one_node(blocks_uu, msh, option)
                     endif
                 enddo
                 do i=kk+1,num_blocks
@@ -470,7 +486,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
                         blocks_ll => h_mat%Local_blocks(myj, myi1)
                         blocks_mm => h_mat%Computing_matricesblock_m(1, 1)
                         call Hmat_XUM(blocks_mm, blocks_ll, h_mat, option, stats, ptree, msh)
-                        call pack_all_blocks_one_node(blocks_ll, msh)
+                        call pack_all_blocks_one_node(blocks_ll, msh, option)
                     endif
                 enddo
                 if (recv == 1)then
@@ -509,7 +525,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
                             if(iproc1==myrow .and. jproc1==mycol)then
                                 blocks_uu => h_mat%Computing_matricesblock_u(myj1, myi1)
                                 call Hmat_block_copy_MPIdata(blocks_uu, blocks_r, msh)
-                                call unpack_all_blocks_one_node(blocks_uu, h_mat%Maxlevel, ptree, msh, mypgno)
+                                call unpack_all_blocks_one_node(blocks_uu, h_mat%Maxlevel, ptree, msh, mypgno, option)
                                 Memory=0
                                 call Hmat_block_ComputeMemory(blocks_uu, Memory)
                                 call LogMemory(stats, Memory)
@@ -542,7 +558,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
                             if(iproc1==myrow .and. jproc1==mycol)then
                                 blocks_ll => h_mat%Computing_matricesblock_l(myj1, myi1)
                                 call Hmat_block_copy_MPIdata(blocks_ll, blocks_r, msh)
-                                call unpack_all_blocks_one_node(blocks_ll, h_mat%Maxlevel, ptree, msh, mypgno)
+                                call unpack_all_blocks_one_node(blocks_ll, h_mat%Maxlevel, ptree, msh, mypgno, option)
                                 Memory=0
                                 call Hmat_block_ComputeMemory(blocks_ll, Memory)
                                 call LogMemory(stats, Memory)
@@ -589,7 +605,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
             do j = 1, h_mat%myAcols
                 do i = 1, h_mat%myArows
                     blocks => h_mat%Local_blocks(j, i)
-                    call unpack_all_blocks_one_node(blocks, h_mat%Maxlevel, ptree, msh, mypgno)
+                    call unpack_all_blocks_one_node(blocks, h_mat%Maxlevel, ptree, msh, mypgno,option)
                     call Hmat_block_ComputeMemory(blocks, stats%Mem_Factor)
                 enddo
             enddo
@@ -850,7 +866,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
         integer style(3), mark, style_m
         integer i, j, k, ii
         integer mm, nn, rank
-        real(kind=8) T0, T1, error
+        real(kind=8) T0, T1, error, tol_used
         type(matrixblock), pointer :: blocks1, blocks2, blocks3
         type(matrixblock) :: blocks_l, blocks_m
         DT:: ctemp
@@ -905,6 +921,12 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
         else
             T0 = MPI_Wtime()
             if (blocks_m%style == 1) then
+#if HAVE_ZFP
+            if(option%use_zfp==1)then
+                 call ZFP_Decompress(blocks_m%fullmat,blocks_m%FullmatZFP,blocks_m%M,blocks_m%N,tol_used,0)
+                 call ZFP_Decompress(blocks_l%fullmat,blocks_l%FullmatZFP,blocks_l%M,blocks_l%N,tol_used,1)
+            endif
+#endif
                 mm = size(blocks_m%fullmat, 1)
                 nn = size(blocks_m%fullmat, 2)
                 do i = 1, mm
@@ -920,9 +942,17 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
                     endif
                 enddo
                 call trsmf90(blocks_l%fullmat, blocks_m%fullmat, 'L', 'L', 'N', 'U', mm, nn)
+#if HAVE_ZFP
+                if(option%use_zfp==1)then
+                    call ZFP_Compress(blocks_m%fullmat,blocks_m%FullmatZFP,blocks_m%M,blocks_m%N,option%tol_comp,0)
+                    call ZFP_Compress(blocks_l%fullmat,blocks_l%FullmatZFP,blocks_l%M,blocks_l%N,option%tol_comp,1)
+                endif
+#endif
+
             else
 
                 write (*, *) 'should not come here H_Solve_XLM'
+                stop
 
                 mm = blocks_m%M
                 rank = size(blocks_m%ButterflyU%blocks(1)%matrix, 2)
@@ -963,7 +993,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
         integer style(3), mark, style_m
         integer i, j, k, mm, nn, rank
         integer mm_1, mm_2, nn_1, nn_2, rank_1, rank_2, mm_3, nn_3, rank_3
-        real(kind=8) T0, T1, error
+        real(kind=8) T0, T1, error, tol_used
         type(matrixblock) :: blocks_u, blocks_m
         type(matrixblock), pointer :: blocks1, blocks2, blocks3
         integer rank0
@@ -1015,12 +1045,25 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
         else
             T0 = MPI_Wtime()
             if (blocks_m%style == 1) then
+#if HAVE_ZFP
+                if(option%use_zfp==1)then
+                    call ZFP_Decompress(blocks_m%fullmat,blocks_m%FullmatZFP,blocks_m%M,blocks_m%N,tol_used,0)
+                    call ZFP_Decompress(blocks_u%fullmat,blocks_u%FullmatZFP,blocks_u%M,blocks_u%N,tol_used,1)
+                endif
+#endif
                 mm = size(blocks_m%fullmat, 1)
                 nn = size(blocks_m%fullmat, 2)
                 call trsmf90(blocks_u%fullmat, blocks_m%fullmat, 'R', 'U', 'N', 'N', mm, nn)
+#if HAVE_ZFP
+                if(option%use_zfp==1)then
+                    call ZFP_Compress(blocks_m%fullmat,blocks_m%FullmatZFP,blocks_m%M,blocks_m%N,option%tol_comp,0)
+                    call ZFP_Compress(blocks_u%fullmat,blocks_u%FullmatZFP,blocks_u%M,blocks_u%N,option%tol_comp,1)
+                endif
+#endif
             else
 
                 write (*, *) 'should not come here Hmat_XUM'
+                stop
                 mm = blocks_m%M
                 rank = size(blocks_m%ButterflyV%blocks(1)%matrix, 2)
                 call trsmf90(blocks_u%fullmat, blocks_m%ButterflyV%blocks(1)%matrix, 'L', 'U', 'T', 'N', mm, rank)
@@ -1213,7 +1256,7 @@ stats%Mem_Direct_inv = stats%Mem_Direct_inv + SIZEOF(ho_bf1%levels(level_c)%BP_i
             stats%Add_random_Time(blocks_o%level) = stats%Add_random_Time(blocks_o%level) + T1 - T0
             stats%Add_random_CNT(blocks_o%level) = stats%Add_random_CNT(blocks_o%level) + 1
         else if (blocks_o%style == 1) then
-            call Full_add(blocks_o, chara, blocks_1, ptree, stats)
+            call Full_add(blocks_o, chara, blocks_1, ptree, stats, option)
         end if
 
     end subroutine Hmat_BF_add
