@@ -22,15 +22,15 @@ module BPACK_Utilities
 contains
 
 
-   recursive subroutine Hmat_GetBlkLst(blocks, ptree, h_mat)
+   recursive subroutine Hmat_GetBlkLst(blocks, ptree, lstblks, Maxlevel)
       implicit none
       ! type(Hoption)::option
       ! type(Hstat)::stats
       ! type(mesh)::msh
       type(proctree)::ptree
-      integer nth, bidx, level_c
+      integer nth, bidx, level_c, Maxlevel
       integer ii, jj, idx, row_group, col_group
-      type(Hmat)::h_mat
+      type(list)::lstblks(0:Maxlevel)
       type(nod), pointer::cur
       class(*), pointer::ptr
       type(matrixblock), pointer::blocks, blocks_son
@@ -39,25 +39,24 @@ contains
 
       row_group = blocks%row_group
       col_group = blocks%col_group
-      if (IOwnPgrp(ptree, blocks%pgno)) then
+      ! if (IOwnPgrp(ptree, blocks%pgno)) then
          if (blocks%style == 4) then ! divided blocks
             do ii = 1, 2
             do jj = 1, 2
                blocks_son => blocks%sons(ii, jj)
-               call Hmat_GetBlkLst(blocks_son, ptree, h_mat)
+               call Hmat_GetBlkLst(blocks_son, ptree, lstblks, Maxlevel)
             enddo
             enddo
          else
             blk_ptr%ptr => blocks
-            call append(h_mat%lstblks(blocks%level), blk_ptr)
+            call append(lstblks(blocks%level), blk_ptr)
          endif
-      endif
+      ! endif
    end subroutine Hmat_GetBlkLst
 
 
    subroutine HODLR_copy(ho_bf_i, ho_bf_o, ptree)
-      use BPACK_DEFS
-      use MISC_Utilities
+
 
       implicit none
 
@@ -105,8 +104,7 @@ contains
 
 
    subroutine Hmat_copy(h_mat_i, h_mat_o, ptree)
-      use BPACK_DEFS
-      use MISC_Utilities
+
       implicit none
       integer ii,jj,bm,bn,level
       type(Hmat)::h_mat_i, h_mat_o
@@ -181,7 +179,7 @@ contains
          do ii = 1, bm
             do jj = 1, bn
                blocks1 => h_mat_o%Local_blocks(ii, jj)
-               call Hmat_GetBlkLst(blocks1, ptree, h_mat_o)
+               call Hmat_GetBlkLst(blocks1, ptree, h_mat_o%lstblks, h_mat_o%Maxlevel)
             enddo
          enddo
          do level = 0, h_mat_o%Maxlevel
@@ -193,8 +191,7 @@ contains
 
 
    subroutine Hmat_delete(h_mat)
-      use BPACK_DEFS
-      use MISC_Utilities
+
       implicit none
 
       type(Hmat)::h_mat
@@ -246,8 +243,7 @@ contains
    end subroutine Hmat_delete
 
    subroutine HODLR_delete(ho_bf_o)
-      use BPACK_DEFS
-      use MISC_Utilities
+
       implicit none
 
       type(hobf)::ho_bf_o
@@ -277,8 +273,7 @@ contains
 
 
    subroutine HSSBF_delete(hss_bf1)
-      use BPACK_DEFS
-      use MISC_Utilities
+
       implicit none
 
       type(hssbf)::hss_bf1
@@ -292,8 +287,44 @@ contains
    end subroutine HSSBF_delete
 
 
+
+   subroutine HSSBF_MD_delete(hss_bf1_md)
+      implicit none
+      integer ll,bb
+      type(hssbf_md)::hss_bf1_md
+
+      if (associated(hss_bf1_md%BP%LL)) then
+         do ll = 1, LplusMax
+            if (hss_bf1_md%BP%LL(ll)%Nbound > 0) then
+               if (associated(hss_bf1_md%BP%LL(ll)%matrices_block)) then
+               do bb = 1, hss_bf1_md%BP%LL(ll)%Nbound
+                  ! write(*,*)ll,hss_bf1_md%BP%Lplus,bb,hss_bf1_md%BP%LL(ll)%Nbound,'fff'
+                  call BF_MD_delete(hss_bf1_md%Ndim, hss_bf1_md%BP%LL(ll)%matrices_block(bb), 1)
+               end do
+               deallocate (hss_bf1_md%BP%LL(ll)%matrices_block)
+               endif
+               if (allocated(hss_bf1_md%BP%LL(ll)%boundary_map)) deallocate (hss_bf1_md%BP%LL(ll)%boundary_map)
+            end if
+         end do
+         deallocate (hss_bf1_md%BP%LL)
+      endif
+
+      if(allocated(hss_bf1_md%BP%row_group))then
+         deallocate(hss_bf1_md%BP%row_group)
+      endif
+      if(allocated(hss_bf1_md%BP%col_group))then
+         deallocate(hss_bf1_md%BP%col_group)
+      endif
+      if(allocated(hss_bf1_md%N))then
+         deallocate(hss_bf1_md%N)
+      endif
+
+   end subroutine HSSBF_MD_delete
+
+
+
    subroutine BPACK_delete(bmat)
-      use BPACK_DEFS
+
       implicit none
       type(Bmatrix)::bmat
       if (associated(bmat%ho_bf)) then
@@ -312,11 +343,17 @@ contains
          bmat%hss_bf => null()
       endif
 
+      if (associated(bmat%hss_bf_md)) then
+         call HSSBF_MD_delete(bmat%hss_bf_md)
+         deallocate (bmat%hss_bf_md)
+         bmat%hss_bf_md => null()
+      endif
+
    end subroutine BPACK_delete
 
 
    subroutine BPACK_copy(bmat_i,bmat_o,ptree)
-      use BPACK_DEFS
+
       implicit none
       type(Bmatrix)::bmat_i,bmat_o
       type(proctree)::ptree
@@ -337,16 +374,14 @@ contains
 
 
    subroutine delete_kernelquant(ker)
-      use BPACK_DEFS
-      use MISC_Utilities
+
       implicit none
       type(kernelquant)::ker
       if (allocated(ker%matZ_glo)) deallocate (ker%matZ_glo)
    end subroutine delete_kernelquant
 
    subroutine delete_mesh(msh,stats)
-      use BPACK_DEFS
-      use MISC_Utilities
+
       implicit none
       type(mesh)::msh
       type(Hstat),optional::stats
@@ -382,8 +417,7 @@ contains
    end subroutine delete_mesh
 
    subroutine delete_proctree(ptree)
-      use BPACK_DEFS
-      use MISC_Utilities
+
       implicit none
       type(proctree)::ptree
       integer ii, Maxgrp
@@ -397,10 +431,10 @@ contains
             ! deallocate(ptree%pgrp(ii)%gd)
             ! ptree%pgrp(ii)%gd=>null()
             ! endif
-            if (ptree%pgrp(ii)%ctxt /= -1) call blacs_gridexit(ptree%pgrp(ii)%ctxt)
-            if (ptree%pgrp(ii)%ctxt1D /= -1) call blacs_gridexit(ptree%pgrp(ii)%ctxt1D)
-            if (ptree%pgrp(ii)%ctxt1DCol /= -1) call blacs_gridexit(ptree%pgrp(ii)%ctxt1DCol)
-            if (ptree%pgrp(ii)%ctxt_head /= -1) call blacs_gridexit(ptree%pgrp(ii)%ctxt_head)
+            if (ptree%pgrp(ii)%ctxt /= -1) call blacs_gridexit_wrp(ptree%pgrp(ii)%ctxt)
+            if (ptree%pgrp(ii)%ctxt1D /= -1) call blacs_gridexit_wrp(ptree%pgrp(ii)%ctxt1D)
+            if (ptree%pgrp(ii)%ctxt1DCol /= -1) call blacs_gridexit_wrp(ptree%pgrp(ii)%ctxt1DCol)
+            if (ptree%pgrp(ii)%ctxt_head /= -1) call blacs_gridexit_wrp(ptree%pgrp(ii)%ctxt_head)
             if (ptree%pgrp(ii)%Comm /= MPI_COMM_NULL) call MPI_Comm_free(ptree%pgrp(ii)%Comm, ierr)
          enddo
          deallocate (ptree%pgrp)
@@ -410,14 +444,13 @@ contains
    end subroutine delete_proctree
 
 ! recursive subroutine delete_grid(gd)
-! use BPACK_DEFS
-! use MISC_Utilities
+
 ! implicit none
 ! type(grid)::gd
 ! integer ierr
 
 ! if(.not. associated(gd%gdc))then
-   ! if(gd%ctxt/=-1)call blacs_gridexit(gd%ctxt)
+   ! if(gd%ctxt/=-1)call blacs_gridexit_wrp(gd%ctxt)
    ! if(gd%Comm/=MPI_COMM_NULL)call MPI_Comm_free(gd%Comm,ierr)
    ! return
 ! else
@@ -429,8 +462,7 @@ contains
 ! end subroutine delete_grid
 
    subroutine delete_Hstat(stats)
-      use BPACK_DEFS
-      use MISC_Utilities
+
       implicit none
       type(Hstat)::stats
 
@@ -457,7 +489,7 @@ contains
 
          basis_group2(node2)%head = basis_group1(node1)%head + offset
          basis_group2(node2)%tail = basis_group1(node1)%tail + offset
-         basis_group2(node2)%pgno = basis_group1(node1)%pgno
+         ! basis_group2(node2)%pgno = basis_group1(node1)%pgno
          basis_group2(node2)%radius = basis_group1(node1)%radius
          if(allocated(basis_group1(node1)%center))then
             Dimn = size(basis_group1(node1)%center, 1)
@@ -719,6 +751,7 @@ contains
 
       call MPI_ALLREDUCE(stats%Mem_peak, rtemp, 1, MPI_DOUBLE_PRECISION, MPI_MAX, ptree%Comm, ierr)
       if (ptree%MyID == Main_ID) write (*, '(A21,Es14.2,A3)') 'Peak mem:', rtemp, 'MB'
+      if (ptree%MyID == Main_ID) write (*, *) 'time_tmp', time_tmp
 
    end subroutine PrintStat
 
@@ -763,10 +796,13 @@ contains
       option%format = HODLR
       option%verbosity = 0
       option%scale_factor = 1d0
+      option%jitter = 1d-13
       option%rmax = 3000
       option%forwardN15flag = 0
+      option%fastsample_tensor = 0
       option%sample_para = 2.0d0
       option%sample_para_outer = 2.0d0
+      option%use_zfp = 0
       ! option%sample_heuristic = 1
       option%pat_comp = 3
       option%elem_extract = 0
@@ -830,6 +866,8 @@ contains
                   read (strings1, *) option%ErrFillFull
                else if (trim(strings) == '--forwardN15flag') then
                   read (strings1, *) option%forwardN15flag
+               else if (trim(strings) == '--fastsample_tensor') then
+                  read (strings1, *) option%fastsample_tensor
                else if (trim(strings) == '--baca_batch') then
                   read (strings1, *) option%BACA_Batch
                else if (trim(strings) == '--reclr_leaf') then
@@ -876,6 +914,8 @@ contains
                   read (strings1, *) option%sample_para
                else if (trim(strings) == '--sample_para_outer') then
                   read (strings1, *) option%sample_para_outer
+               else if (trim(strings) == '--use_zfp') then
+                  read (strings1, *) option%use_zfp
                ! else if (trim(strings) == '--sample_heuristic') then
                !    read (strings1, *) option%sample_heuristic
                else if (trim(strings) == '--pat_comp') then
@@ -947,10 +987,13 @@ contains
       option1%format = option%format
       option1%verbosity = option%verbosity
       option1%scale_factor = option%scale_factor
+      option1%jitter = option%jitter
       option1%rmax = option%rmax
       option1%forwardN15flag = option%forwardN15flag
+option1%fastsample_tensor = option%fastsample_tensor
       option1%sample_para = option%sample_para
       option1%sample_para_outer = option%sample_para_outer
+      option1%use_zfp = option%use_zfp
       ! option1%sample_heuristic = option%sample_heuristic
       option1%pat_comp = option%pat_comp
       option1%elem_extract = option%elem_extract
@@ -969,7 +1012,8 @@ contains
          write (*, *) ' '
          write (*, *) '***************************'
          write (*, '(A25)') 'Printing Solver Options:'
-         write (*, '(A18,I8)') 'Nmin_leaf', option%Nmin_leaf
+         write (*, '(A18,I8)') 'format', option%format
+         write (*, '(A18,I8)') 'nmin_leaf', option%Nmin_leaf
          write (*, '(A18,I8)') 'n_iter', option%n_iter
          write (*, '(A18,I8)') 'level_check', option%level_check
          write (*, '(A18,I8)') 'precon', option%precon
@@ -981,31 +1025,33 @@ contains
          write (*, '(A18,I8)') 'schulzhardstart', option%schulzhardstart
          write (*, '(A18,I8)') 'schulzsplitlevel', option%schulzsplitlevel
          write (*, '(A18,I8)') 'schulzlevel', option%schulzlevel
-         write (*, '(A18,I8)') 'LRlevel', option%LRlevel
-         write (*, '(A18,I8)') 'BACA_Batch', option%BACA_Batch
-         write (*, '(A18,I8)') 'RecLR_leaf', option%RecLR_leaf
+         write (*, '(A18,I8)') 'lrlevel', option%LRlevel
+         write (*, '(A18,I8)') 'baca_batch', option%BACA_Batch
+         write (*, '(A18,I8)') 'reclr_leaf', option%RecLR_leaf
          write (*, '(A18,I8)') 'nogeo', option%nogeo
          write (*, '(A18,I8)') 'per_geo', option%per_geo
          write (*, '(A18,Es14.7)') 'period1', option%periods(1)
          write (*, '(A18,Es14.7)') 'period2', option%periods(2)
          write (*, '(A18,Es14.7)') 'period3', option%periods(3)
-         write (*, '(A18,I8)') 'LR_BLK_NUM', option%LR_BLK_NUM
+         write (*, '(A18,I8)') 'lr_blk_num', option%LR_BLK_NUM
          write (*, '(A18,I8)') 'rank0', option%rank0
          write (*, '(A18,I8)') 'itermax', option%itermax
          write (*, '(A18,I8)') 'powiter', option%powiter
-         write (*, '(A18,I8)') 'ILU', option%ILU
-         write (*, '(A18,I8)') 'Nbundle', option%Nbundle
+         write (*, '(A18,I8)') 'ilu', option%ILU
+         write (*, '(A18,I8)') 'nbundle', option%Nbundle
          write (*, '(A18,I8)') 'verbosity', option%verbosity
          write (*, '(A18,I8)') 'rmax', option%rmax
          write (*, '(A18,I8)') 'forwardN15flag', option%forwardN15flag
+         write (*, '(A18,I8)') 'fastsample_tensor', option%fastsample_tensor
          write (*, '(A18,I8)') 'pat_comp', option%pat_comp
          write (*, '(A18,I8)') 'elem_extract', option%elem_extract
          write (*, '(A18,I8)') 'cpp', option%cpp
          write (*, '(A18,I8)') 'knn', option%knn
-         write (*, '(A18,I8)') 'ErrFillFull', option%ErrFillFull
-         write (*, '(A18,I8)') 'ErrSol', option%ErrSol
+         write (*, '(A18,I8)') 'errfillfull', option%ErrFillFull
+         write (*, '(A18,I8)') 'errsol', option%ErrSol
          ! write (*, '(A18,I8)') 'sample_heuristic', option%sample_heuristic
          write (*, '(A18,I8)') 'less_adapt', option%less_adapt
+         write (*, '(A18,I8)') 'use_zfp', option%use_zfp
 
          write (*, '(A18,Es14.7)') 'rankrate', option%rankrate
          write (*, '(A18,Es14.7)') 'tol_comp', option%tol_comp
@@ -1017,6 +1063,7 @@ contains
          write (*, '(A18,Es14.7)') 'near_para', option%near_para
          write (*, '(A18,Es14.7)') 'knn_near_para', option%knn_near_para
          write (*, '(A18,Es14.7)') 'scale_factor', option%scale_factor
+         write (*, '(A18,Es14.7)') 'jitter_factor', option%jitter
          write (*, '(A18,Es14.7)') 'sample_para', option%sample_para
          write (*, '(A18,Es14.7)') 'sample_para_outer', option%sample_para_outer
          write (*, *) '***************************'
@@ -1025,7 +1072,6 @@ contains
    end subroutine PrintOptions
 
    subroutine BPACK_GetVersionNumber(v_major, v_minor, v_bugfix)
-      use BPACK_DEFS
       implicit none
       integer v_major, v_minor, v_bugfix
       v_major = BPACK_MAJOR_VERSION
